@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2017 KeePassXC Team <team@keepassxc.org>
+ *  Copyright (C) 2019 KeePassXC Team <team@keepassxc.org>
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,8 +24,9 @@ class Estimate : public Command
 {
 public:
     Estimate();
-    ~Estimate();
     int execute(const QStringList& arguments) override;
+
+    static const QCommandLineOption AdvancedOption;
 };
 
 #endif // KEEPASSXC_ESTIMATE_H
