@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2017 KeePassXC Team <team@keepassxc.org>
+ *  Copyright (C) 2019 KeePassXC Team <team@keepassxc.org>
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,15 +18,17 @@
 #ifndef KEEPASSXC_LIST_H
 #define KEEPASSXC_LIST_H
 
-#include "Command.h"
+#include "DatabaseCommand.h"
 
-class List : public Command
+class List : public DatabaseCommand
 {
 public:
     List();
-    ~List();
-    int execute(const QStringList& arguments) override;
-    int listGroup(Database* database, bool recursive, const QString& groupPath = {});
+
+    int executeWithDatabase(QSharedPointer<Database> db, QSharedPointer<QCommandLineParser> parser) override;
+
+    static const QCommandLineOption RecursiveOption;
+    static const QCommandLineOption FlattenOption;
 };
 
 #endif // KEEPASSXC_LIST_H
