@@ -20,6 +20,7 @@
 
 #include "core/Config.h"
 #include "core/Database.h"
+#include "core/Global.h"
 #include "core/Group.h"
 #include "core/Metadata.h"
 #include "core/PasswordHealth.h"
@@ -292,7 +293,7 @@ QString Entry::effectiveAutoTypeSequence() const
 
 /**
  * Retrieve the Auto-Type sequences matches for a given windowTitle
- * This returns a list with priority ordering. If you don't want duplicates call .toSet() on it.
+ * This returns a list with priority ordering. If you don't want duplicates, convert it to a QSet<QString>.
  */
 QList<QString> Entry::autoTypeSequences(const QString& windowTitle) const
 {
@@ -333,12 +334,15 @@ QList<QString> Entry::autoTypeSequences(const QString& windowTitle) const
     };
 
     QList<QString> sequenceList;
+    QList<QString> emptyWindowSequences;
 
     // Add window association matches
     const auto assocList = autoTypeAssociations()->getAll();
     for (const auto& assoc : assocList) {
         auto window = resolveMultiplePlaceholders(assoc.window);
-        if (!assoc.window.isEmpty() && windowMatches(window)) {
+        if (assoc.window.isEmpty()) {
+            emptyWindowSequences << assoc.sequence;
+        } else if (windowMatches(window)) {
             if (!assoc.sequence.isEmpty()) {
                 sequenceList << assoc.sequence;
             } else {
@@ -357,6 +361,11 @@ QList<QString> Entry::autoTypeSequences(const QString& windowTitle) const
         sequenceList << effectiveAutoTypeSequence();
     }
 
+    // If any associations were made, include the empty window associations
+    if (!sequenceList.isEmpty()) {
+        sequenceList.append(emptyWindowSequences);
+    }
+
     return sequenceList;
 }
 
@@ -458,7 +467,7 @@ int Entry::size() const
     size += autoTypeAssociations()->associationsSize();
     size += attachments()->attachmentsSize();
     size += customData()->dataSize();
-    for (const QString& tag : tags().split(TagDelimiterRegex, QString::SkipEmptyParts)) {
+    for (const QString& tag : tags().split(TagDelimiterRegex, Qt::SkipEmptyParts)) {
         size += tag.toUtf8().size();
     }
 
@@ -569,6 +578,12 @@ bool Entry::hasTotp() const
     return !m_data.totpSettings.isNull();
 }
 
+bool Entry::hasValidTotp() const
+{
+    auto error = Totp::checkValidSettings(m_data.totpSettings);
+    return error.isEmpty();
+}
+
 bool Entry::hasPasskey() const
 {
     return m_attributes->hasPasskey();
@@ -580,10 +595,13 @@ void Entry::removePasskey()
     removeTag(tr("Passkey"));
 }
 
-QString Entry::totp() const
+QString Entry::totp(bool* isValid) const
 {
     if (hasTotp()) {
-        return Totp::generateTotp(m_data.totpSettings);
+        return Totp::generateTotp(m_data.totpSettings, isValid);
+    }
+    if (isValid) {
+        *isValid = false;
     }
     return {};
 }
@@ -698,14 +716,13 @@ void Entry::setOverrideUrl(const QString& url)
 
 void Entry::setTags(const QString& tags)
 {
-    auto taglist = tags.split(TagDelimiterRegex, QString::SkipEmptyParts);
+    auto taglist = tags.split(TagDelimiterRegex, Qt::SkipEmptyParts);
     // Trim whitespace before/after tag text
     for (auto& tag : taglist) {
         tag = tag.trimmed();
     }
     // Remove duplicates
-    auto tagSet = QSet<QString>::fromList(taglist);
-    taglist = tagSet.toList();
+    taglist = Tools::asSet(taglist).values();
     // Sort alphabetically
     taglist.sort();
     set(m_data.tags, taglist);
@@ -941,6 +958,68 @@ bool Entry::equals(const Entry* other, CompareItemOptions options) const
     return true;
 }
 
+QStringList Entry::calculateDifference(const Entry* other)
+{
+    QStringList modifiedFields;
+
+    if (*attributes() != *other->attributes()) {
+        bool foundAttribute = false;
+
+        if (title() != other->title()) {
+            modifiedFields << tr("Title");
+            foundAttribute = true;
+        }
+        if (username() != other->username()) {
+            modifiedFields << tr("Username");
+            foundAttribute = true;
+        }
+        if (password() != other->password()) {
+            modifiedFields << tr("Password");
+            foundAttribute = true;
+        }
+        if (url() != other->url()) {
+            modifiedFields << tr("URL");
+            foundAttribute = true;
+        }
+        if (notes() != other->notes()) {
+            modifiedFields << tr("Notes");
+            foundAttribute = true;
+        }
+
+        if (!foundAttribute) {
+            modifiedFields << tr("Custom Attributes");
+        }
+    }
+    if (iconNumber() != other->iconNumber() || iconUuid() != other->iconUuid()) {
+        modifiedFields << tr("Icon");
+    }
+    if (foregroundColor() != other->foregroundColor() || backgroundColor() != other->backgroundColor()) {
+        modifiedFields << tr("Color");
+    }
+    if (timeInfo().expires() != other->timeInfo().expires()
+        || timeInfo().expiryTime() != other->timeInfo().expiryTime()) {
+        modifiedFields << tr("Expiration");
+    }
+    if (totp() != other->totp()) {
+        modifiedFields << tr("TOTP");
+    }
+    if (*customData() != *other->customData()) {
+        modifiedFields << tr("Custom Data");
+    }
+    if (*attachments() != *other->attachments()) {
+        modifiedFields << tr("Attachments");
+    }
+    if (*autoTypeAssociations() != *other->autoTypeAssociations() || autoTypeEnabled() != other->autoTypeEnabled()
+        || defaultAutoTypeSequence() != other->defaultAutoTypeSequence()) {
+        modifiedFields << tr("Auto-Type");
+    }
+    if (tags() != other->tags()) {
+        modifiedFields << tr("Tags");
+    }
+
+    return modifiedFields;
+}
+
 Entry* Entry::clone(CloneFlags flags) const
 {
     Entry* entry = new Entry();
@@ -1050,6 +1129,15 @@ QString Entry::resolveMultiplePlaceholdersRecursive(const QString& str, int maxD
         return str;
     }
 
+    // Short circuit if we have escaped the placeholder brackets
+    if (str.startsWith("\\{") && str.endsWith("\\}")) {
+        // Replace the escaped brackets with actuals and move on
+        auto ret = str;
+        ret.replace(0, 2, "{");
+        ret.replace(ret.size() - 2, 2, "}");
+        return ret;
+    }
+
     QString result;
     auto matches = placeholderRegEx.globalMatch(str);
     int capEnd = 0;
@@ -1087,6 +1175,8 @@ QString Entry::resolvePlaceholderRecursive(const QString& placeholder, int maxDe
         return resolveMultiplePlaceholdersRecursive(notes(), maxDepth);
     case PlaceholderType::Url:
         return resolveMultiplePlaceholdersRecursive(url(), maxDepth);
+    case PlaceholderType::Uuid:
+        return uuidToHex();
     case PlaceholderType::DbDir: {
         QFileInfo fileInfo(database()->filePath());
         return fileInfo.absoluteDir().absolutePath();
@@ -1282,7 +1372,10 @@ QString Entry::resolveReferencePlaceholderRecursive(const QString& placeholder,
 
     QString result;
     const QString searchIn = match.captured(EntryAttributes::SearchInGroupName);
-    const QString searchText = match.captured(EntryAttributes::SearchTextGroupName);
+    QString searchText = match.captured(EntryAttributes::SearchTextGroupName);
+
+    // Resolve placeholders in the search text (e.g., {UUID} -> actual UUID)
+    searchText = resolvePlaceholder(searchText);
 
     const EntryReferenceType searchInType = Entry::referenceType(searchIn);
 
@@ -1493,6 +1586,7 @@ Entry::PlaceholderType Entry::placeholderType(const QString& placeholder) const
         {QStringLiteral("{NOTES}"), PlaceholderType::Notes},
         {QStringLiteral("{TOTP}"), PlaceholderType::Totp},
         {QStringLiteral("{URL}"), PlaceholderType::Url},
+        {QStringLiteral("{UUID}"), PlaceholderType::Uuid},
         {QStringLiteral("{URL:RMVSCM}"), PlaceholderType::UrlWithoutScheme},
         {QStringLiteral("{URL:WITHOUTSCHEME}"), PlaceholderType::UrlWithoutScheme},
         {QStringLiteral("{URL:SCM}"), PlaceholderType::UrlScheme},
