@@ -31,12 +31,52 @@ public:
     Merger(const Group* sourceGroup, Group* targetGroup);
     void setForcedMergeMode(Group::MergeMode mode);
     void resetForcedMergeMode();
-    QStringList merge();
+    void setSkipDatabaseCustomData(bool state);
+
+    class Change
+    {
+    public:
+        enum class Type
+        {
+            Unspecified,
+            Added,
+            Modified,
+            Moved,
+            Deleted,
+            Metadata,
+        };
+
+        Change(Type type, QString details);
+        Change(Type type, const Group& group, QString details = "");
+        Change(Type type, const Entry& entry, QString details = "");
+        explicit Change(QString details = "");
+
+        [[nodiscard]] Type type() const;
+        [[nodiscard]] QString typeString() const;
+        [[nodiscard]] const QString& title() const;
+        [[nodiscard]] const QString& group() const;
+        [[nodiscard]] const QUuid& uuid() const;
+        [[nodiscard]] const QString& details() const;
+
+        [[nodiscard]] QString toString() const;
+        void merge();
+
+        bool operator==(const Change& other) const;
+        bool operator!=(const Change& other) const;
+
+    private:
+        Type m_type{Type::Unspecified};
+        QString m_title;
+        QString m_group;
+        QUuid m_uuid;
+        QString m_details;
+    };
+
+    using ChangeList = QList<Change>;
+
+    ChangeList merge(bool dryRun = false);
 
 private:
-    typedef QString Change;
-    typedef QStringList ChangeList;
-
     struct MergeContext
     {
         QPointer<const Database> m_sourceDb;
@@ -46,11 +86,11 @@ private:
         QPointer<const Group> m_sourceGroup;
         QPointer<Group> m_targetGroup;
     };
+
     ChangeList mergeGroup(const MergeContext& context);
     ChangeList mergeDeletions(const MergeContext& context);
     ChangeList mergeMetadata(const MergeContext& context);
-    bool markOlderEntry(Entry* entry);
-    bool mergeHistory(const Entry* sourceEntry, Entry* targetEntry, Group::MergeMode mergeMethod);
+    bool mergeHistory(const Entry* sourceEntry, Entry* targetEntry, Group::MergeMode mergeMethod, const int maxItems);
     void moveEntry(Entry* entry, Group* targetGroup);
     void moveGroup(Group* group, Group* targetGroup);
     // remove an entry without a trace in the deletedObjects - needed for elemination cloned entries
@@ -59,12 +99,6 @@ private:
     void eraseGroup(Group* group);
     ChangeList resolveEntryConflict(const MergeContext& context, const Entry* existingEntry, Entry* otherEntry);
     ChangeList resolveGroupConflict(const MergeContext& context, const Group* existingGroup, Group* otherGroup);
-    Merger::ChangeList
-    resolveEntryConflict_Duplicate(const MergeContext& context, const Entry* sourceEntry, Entry* targetEntry);
-    Merger::ChangeList
-    resolveEntryConflict_KeepLocal(const MergeContext& context, const Entry* sourceEntry, Entry* targetEntry);
-    Merger::ChangeList
-    resolveEntryConflict_KeepRemote(const MergeContext& context, const Entry* sourceEntry, Entry* targetEntry);
     Merger::ChangeList resolveEntryConflict_MergeHistories(const MergeContext& context,
                                                            const Entry* sourceEntry,
                                                            Entry* targetEntry,
@@ -73,6 +107,8 @@ private:
 private:
     MergeContext m_context;
     Group::MergeMode m_mode;
+    bool m_skipCustomData = false;
+    bool m_dryRun = false;
 };
 
 #endif // KEEPASSXC_MERGER_H
