@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2023 KeePassXC Team <team@keepassxc.org>
+ *  Copyright (C) 2024 KeePassXC Team <team@keepassxc.org>
  *
  *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -56,6 +56,7 @@ namespace
 
         // Create entry and assign basic values
         QScopedPointer<Entry> entry(new Entry());
+        entry->setEmitModified(false);
         entry->setUuid(QUuid::createUuid());
         entry->setTitle(itemMap.value("name").toString());
         entry->setNotes(itemMap.value("notes").toString());
@@ -81,6 +82,43 @@ namespace
                 entry->setTotp(Totp::parseSettings(totp));
             }
 
+            // Parse passkey
+            if (loginMap.contains("fido2Credentials")) {
+                const auto fido2CredentialsMap = loginMap.value("fido2Credentials").toList();
+                for (const auto& fido2Credentials : fido2CredentialsMap) {
+                    const auto passkey = fido2Credentials.toMap();
+
+                    // Change from UUID to base64 byte array
+                    const auto credentialIdValue = passkey.value("credentialId").toString();
+                    if (!credentialIdValue.isEmpty()) {
+                        const auto credentialUuid = Tools::uuidToHex(credentialIdValue);
+                        const auto credentialIdArray = QByteArray::fromHex(credentialUuid.toUtf8());
+                        const auto credentialId =
+                            credentialIdArray.toBase64(QByteArray::Base64UrlEncoding | QByteArray::OmitTrailingEquals);
+                        entry->attributes()->set(EntryAttributes::KPEX_PASSKEY_CREDENTIAL_ID, credentialId, true);
+                    }
+
+                    // Base64 needs to be changed from URL encoding back to normal, and the result as PEM string
+                    const auto keyValue = passkey.value("keyValue").toString();
+                    if (!keyValue.isEmpty()) {
+                        const auto keyValueArray =
+                            QByteArray::fromBase64(keyValue.toUtf8(), QByteArray::Base64UrlEncoding);
+                        auto privateKey = keyValueArray.toBase64(QByteArray::Base64Encoding);
+                        privateKey.insert(0, EntryAttributes::KPEX_PASSKEY_PRIVATE_KEY_START.toUtf8());
+                        privateKey.append(EntryAttributes::KPEX_PASSKEY_PRIVATE_KEY_END.toUtf8());
+                        entry->attributes()->set(EntryAttributes::KPEX_PASSKEY_PRIVATE_KEY_PEM, privateKey, true);
+                    }
+
+                    entry->attributes()->set(EntryAttributes::KPEX_PASSKEY_USERNAME,
+                                             passkey.value("userName").toString());
+                    entry->attributes()->set(EntryAttributes::KPEX_PASSKEY_RELYING_PARTY,
+                                             passkey.value("rpId").toString());
+                    entry->attributes()->set(
+                        EntryAttributes::KPEX_PASSKEY_USER_HANDLE, passkey.value("userHandle").toString(), true);
+                    entry->addTag(QObject::tr("Passkey"));
+                }
+            }
+
             // Set the entry url(s)
             int i = 1;
             for (const auto& urlObj : loginMap.value("uris").toList()) {
@@ -168,9 +206,58 @@ namespace
             entry->attributes()->set(name, value, type == 1);
         }
 
+        // Parse timestamps
+        auto timeInfo = entry->timeInfo();
+        if (itemMap.contains("creationDate")) {
+            const auto creationDate = QDateTime::fromString(itemMap.value("creationDate").toString(), Qt::ISODate);
+            if (creationDate.isValid()) {
+                timeInfo.setCreationTime(creationDate);
+            }
+        }
+        if (itemMap.contains("revisionDate")) {
+            const auto revisionDate = QDateTime::fromString(itemMap.value("revisionDate").toString(), Qt::ISODate);
+            if (revisionDate.isValid()) {
+                timeInfo.setLastModificationTime(revisionDate);
+                timeInfo.setLastAccessTime(revisionDate);
+            }
+        }
+        entry->setTimeInfo(timeInfo);
+
         // Collapse any accumulated history
         entry->removeHistoryItems(entry->historyItems());
 
+        // Parse password history, if present
+        if (itemMap.contains("passwordHistory")) {
+            const auto passwordHistory = itemMap.value("passwordHistory").toList();
+            for (const auto& historyItem : passwordHistory) {
+                const auto historyMap = historyItem.toMap();
+                const auto password = historyMap.value("password").toString();
+                const auto lastUsedDate =
+                    QDateTime::fromString(historyMap.value("lastUsedDate").toString(), Qt::ISODate);
+
+                if (!password.isEmpty() && lastUsedDate.isValid()) {
+                    // Create a history entry with the old password
+                    auto historyEntry = new Entry();
+                    historyEntry->setUuid(entry->uuid());
+                    historyEntry->setTitle(entry->title());
+                    historyEntry->setUsername(entry->username());
+                    historyEntry->setPassword(password);
+                    historyEntry->setUrl(entry->url());
+                    historyEntry->setNotes(entry->notes());
+
+                    // Set the timestamp for this history item
+                    auto historyTimeInfo = historyEntry->timeInfo();
+                    historyTimeInfo.setCreationTime(entry->timeInfo().creationTime());
+                    historyTimeInfo.setLastModificationTime(lastUsedDate);
+                    historyTimeInfo.setLastAccessTime(lastUsedDate);
+                    historyEntry->setTimeInfo(historyTimeInfo);
+
+                    entry->addHistoryItem(historyEntry);
+                }
+            }
+        }
+
+        entry->setEmitModified(true);
         return entry.take();
     }
 
@@ -203,7 +290,9 @@ namespace
         for (const auto& item : items) {
             auto entry = readItem(item.toObject(), folderId);
             if (entry) {
+                entry->setUpdateTimeinfo(false);
                 entry->setGroup(folderMap.value(folderId, db->rootGroup()), false);
+                entry->setUpdateTimeinfo(true);
             }
         }
     }
