@@ -48,12 +48,14 @@ SearchWidget::SearchWidget(QWidget* parent)
     new QShortcut(Qt::CTRL + Qt::Key_J, this, SLOT(toggleHelp()), nullptr, Qt::WidgetWithChildrenShortcut);
 
     connect(m_ui->searchEdit, SIGNAL(textChanged(QString)), SLOT(startSearchTimer()));
+    connect(m_ui->searchEdit, SIGNAL(textChanged(QString)), SLOT(updateSaveButtonVisibility()));
     connect(m_ui->helpIcon, SIGNAL(triggered()), SLOT(toggleHelp()));
     connect(m_ui->searchIcon, SIGNAL(triggered()), SLOT(showSearchMenu()));
     connect(m_ui->saveIcon, &QAction::triggered, this, [this] { emit saveSearch(m_ui->searchEdit->text()); });
     connect(m_searchTimer, SIGNAL(timeout()), SLOT(startSearch()));
     connect(m_clearSearchTimer, SIGNAL(timeout()), SLOT(clearSearch()));
     connect(this, SIGNAL(escapePressed()), SLOT(clearSearch()));
+    connect(m_ui->searchEdit, &QLineEdit::returnPressed, this, &SearchWidget::onReturnPressed);
 
     m_ui->searchEdit->setPlaceholderText(tr("Search (%1)…", "Search placeholder text, %1 is the keyboard shortcut")
                                              .arg(QKeySequence(QKeySequence::Find).toString(QKeySequence::NativeText)));
@@ -69,6 +71,12 @@ SearchWidget::SearchWidget(QWidget* parent)
     m_actionLimitGroup->setCheckable(true);
     m_actionLimitGroup->setChecked(config()->get(Config::SearchLimitGroup).toBool());
 
+    m_actionWaitForEnter = m_searchMenu->addAction(
+        tr("Press Enter to search"), this, [](bool state) { config()->set(Config::GUI_SearchWaitForEnter, state); });
+    m_actionWaitForEnter->setObjectName("actionSearchWaitForEnter");
+    m_actionWaitForEnter->setCheckable(true);
+    m_actionWaitForEnter->setChecked(config()->get(Config::GUI_SearchWaitForEnter).toBool());
+
     m_ui->searchIcon->setIcon(icons()->icon("system-search"));
     m_ui->searchEdit->addAction(m_ui->searchIcon, QLineEdit::LeadingPosition);
 
@@ -150,17 +158,17 @@ void SearchWidget::connectSignals(SignalMultiplexer& mx)
     mx.connect(this, SIGNAL(caseSensitiveChanged(bool)), SLOT(setSearchCaseSensitive(bool)));
     mx.connect(this, SIGNAL(limitGroupChanged(bool)), SLOT(setSearchLimitGroup(bool)));
     mx.connect(this, SIGNAL(downPressed()), SLOT(focusOnEntries()));
-    mx.connect(SIGNAL(requestSearch(QString)), m_ui->searchEdit, SLOT(setText(QString)));
+    mx.connect(SIGNAL(requestSearch(QString)), this, SLOT(performRequestedSearch(QString)));
     mx.connect(SIGNAL(clearSearch()), this, SLOT(clearSearch()));
     mx.connect(SIGNAL(entrySelectionChanged()), this, SLOT(resetSearchClearTimer()));
     mx.connect(SIGNAL(currentModeChanged(DatabaseWidget::Mode)), this, SLOT(resetSearchClearTimer()));
     mx.connect(SIGNAL(databaseUnlocked()), this, SLOT(focusSearch()));
-    mx.connect(m_ui->searchEdit, SIGNAL(returnPressed()), SLOT(switchToEntryEdit()));
+    mx.connect(this, SIGNAL(enterPressed()), SLOT(switchToEntryEdit()));
 }
 
 void SearchWidget::databaseChanged(DatabaseWidget* dbWidget)
 {
-    if (dbWidget != nullptr) {
+    if (dbWidget) {
         // Set current search text from this database
         m_ui->searchEdit->setText(dbWidget->getCurrentSearch());
         // Enforce search policy
@@ -173,18 +181,15 @@ void SearchWidget::databaseChanged(DatabaseWidget* dbWidget)
 
 void SearchWidget::startSearchTimer()
 {
-    if (!m_searchTimer->isActive()) {
+    if (m_actionWaitForEnter->isChecked()) {
         m_searchTimer->stop();
+    } else {
+        m_searchTimer->start(500);
     }
-    m_searchTimer->start(100);
 }
 
 void SearchWidget::startSearch()
 {
-    if (!m_searchTimer->isActive()) {
-        m_searchTimer->stop();
-    }
-
     m_ui->saveIcon->setVisible(true);
     search(m_ui->searchEdit->text());
 }
@@ -202,18 +207,18 @@ void SearchWidget::updateCaseSensitive()
     emit caseSensitiveChanged(m_actionCaseSensitive->isChecked());
 }
 
-void SearchWidget::setCaseSensitive(bool state)
-{
-    m_actionCaseSensitive->setChecked(state);
-    updateCaseSensitive();
-}
-
 void SearchWidget::updateLimitGroup()
 {
     config()->set(Config::SearchLimitGroup, m_actionLimitGroup->isChecked());
     emit limitGroupChanged(m_actionLimitGroup->isChecked());
 }
 
+void SearchWidget::setCaseSensitive(bool state)
+{
+    m_actionCaseSensitive->setChecked(state);
+    updateCaseSensitive();
+}
+
 void SearchWidget::setLimitGroup(bool state)
 {
     m_actionLimitGroup->setChecked(state);
@@ -246,3 +251,28 @@ void SearchWidget::showSearchMenu()
 {
     m_searchMenu->exec(m_ui->searchEdit->mapToGlobal(m_ui->searchEdit->rect().bottomLeft()));
 }
+
+void SearchWidget::onReturnPressed()
+{
+    if (m_actionWaitForEnter->isChecked()) {
+        m_ui->saveIcon->setVisible(true);
+        emit search(m_ui->searchEdit->text());
+    } else {
+        emit enterPressed();
+    }
+}
+
+void SearchWidget::performRequestedSearch(const QString& text)
+{
+    // This method handles saved searches - it should set the text and immediately trigger search
+    // without any delay, regardless of the "Press Enter to search" setting
+    m_ui->searchEdit->setText(text);
+    m_ui->saveIcon->setVisible(!text.isEmpty());
+    emit search(text);
+}
+
+void SearchWidget::updateSaveButtonVisibility()
+{
+    // Show save button whenever there's non-empty text in the search field
+    m_ui->saveIcon->setVisible(!m_ui->searchEdit->text().isEmpty());
+}
