@@ -39,6 +39,10 @@ private slots:
     void testExport();
     void testEmptyDatabase();
     void testNestedGroups();
+    void testRoundTripWithCustomRootName();
+    void testRoundTripWithDefaultRootName();
+    void testSingleLevelGroup();
+    void testAbsolutePaths();
 
 private:
     QSharedPointer<Database> m_db;
