@@ -18,24 +18,16 @@
 #ifndef KEEPASSX_TESTRANDOMGENERATOR_H
 #define KEEPASSX_TESTRANDOMGENERATOR_H
 
-#include "crypto/Random.h"
-
 #include <QObject>
 
-class RandomBackendPreset;
-
 class TestRandomGenerator : public QObject
 {
     Q_OBJECT
 
 private slots:
-    void initTestCase();
-    void cleanupTestCase();
+    void testArray();
     void testUInt();
     void testUIntRange();
-
-private:
-    RandomBackendPreset* m_backend;
 };
 
 #endif // KEEPASSX_TESTRANDOMGENERATOR_H
