@@ -61,12 +61,12 @@ private slots:
     void testSaveBackupPath();
     void testSaveBackupPath_data();
     void testDatabaseSettings();
-    void testKeePass1Import();
     void testDatabaseLocking();
     void testDragAndDropKdbxFiles();
     void testSortGroups();
     void testAutoType();
     void testTrayRestoreHide();
+    void testMenuActionStates();
 
 private:
     void addCannedEntries();
@@ -82,7 +82,7 @@ private:
     void clickIndex(const QModelIndex& index,
                     QAbstractItemView* view,
                     Qt::MouseButton button,
-                    Qt::KeyboardModifiers stateKey = 0);
+                    Qt::KeyboardModifiers stateKey = {});
     void checkSaveDatabase();
     void checkStatusBarText(const QString& textFragment);
 
